(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
	typeof define === 'function' && define.amd ? define(['exports'], factory) :
	(factory((global.Xethya = global.Xethya || {}, global.Xethya.Utils = {})));
}(this, (function (exports) { 'use strict';

function AwaitValue(value) {
  this.value = value;
}

function AsyncGenerator(gen) {
  var front, back;

  function send(key, arg) {
    return new Promise(function (resolve, reject) {
      var request = {
        key: key,
        arg: arg,
        resolve: resolve,
        reject: reject,
        next: null
      };

      if (back) {
        back = back.next = request;
      } else {
        front = back = request;
        resume(key, arg);
      }
    });
  }

  function resume(key, arg) {
    try {
      var result = gen[key](arg);
      var value = result.value;

      if (value instanceof AwaitValue) {
        Promise.resolve(value.value).then(function (arg) {
          resume("next", arg);
        }, function (arg) {
          resume("throw", arg);
        });
      } else {
        settle(result.done ? "return" : "normal", result.value);
      }
    } catch (err) {
      settle("throw", err);
    }
  }

  function settle(type, value) {
    switch (type) {
      case "return":
        front.resolve({
          value: value,
          done: true
        });
        break;

      case "throw":
        front.reject(value);
        break;

      default:
        front.resolve({
          value: value,
          done: false
        });
        break;
    }

    front = front.next;

    if (front) {
      resume(front.key, front.arg);
    } else {
      back = null;
    }
  }

  this._invoke = send;

  if (typeof gen.return !== "function") {
    this.return = undefined;
  }
}

if (typeof Symbol === "function" && Symbol.asyncIterator) {
  AsyncGenerator.prototype[Symbol.asyncIterator] = function () {
    return this;
  };
}

AsyncGenerator.prototype.next = function (arg) {
  return this._invoke("next", arg);
};

AsyncGenerator.prototype.throw = function (arg) {
  return this._invoke("throw", arg);
};

AsyncGenerator.prototype.return = function (arg) {
  return this._invoke("return", arg);
};







function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

var classCallCheck = _classCallCheck;

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

var createClass = _createClass;

/**
 * @module xethya-extension-range
 * @namespace Xethya.Utils
 */

/**
 * Represents a range of number, described through boundaries.
 *
 * @property {Number} lowerBound - The minimum number in this range.
 * @property {Number} upperBound - The maximum number in this range.
 */
var Range =
/*#__PURE__*/
function () {
  /**
   * Instantiates a Range object.
   * 
   * @constructor
   * @param  {Number} lowerBound - The minimum number in this range.
   * @param  {Number} upperBound - The maximum number in this range.
   * @throws {Error} If lowerBound and upperBound are the same.
   */
  function Range(lowerBound, upperBound) {
    classCallCheck(this, Range);

    if (typeof lowerBound === 'undefined') {
      throw new Error('Range#constructor: lowerBound is required');
    }

    if (typeof lowerBound !== 'number') {
      throw new Error('Range#constructor: lowerBound must be a Number');
    }

    if (typeof upperBound === 'undefined') {
      throw new Error('Range#constructor: upperBound is required');
    }

    if (typeof upperBound !== 'number') {
      throw new Error('Range#constructor: upperBound must be a Number');
    }

    if (lowerBound === upperBound) {
      throw new Error('Range#constructor: lowerBound and upperBound cannot be equal');
    }

    this.lowerBound = Math.min(lowerBound, upperBound);
    this.upperBound = Math.max(lowerBound, upperBound);
  }
  /**
   * Checks if a value is in the defined range.
   *
   * @public
   * @function
   * @param  {Number} value - Value to compare.
   * @return {Boolean} true if in range, false otherwise.
   */


  createClass(Range, [{
    key: "includes",
    value: function includes(value) {
      if (typeof value === 'undefined') {
        throw new Error('Range#includes: value is required');
      }

      if (typeof value !== 'number') {
        throw new Error('Range#includes: value must be a Number');
      }

      return this.lowerBound <= value && value <= this.upperBound;
    }
    /**
     * Converts the Range object to a string representation.
     * 
     * @public
     * @function
     * @return {String}
     */

  }, {
    key: "toString",
    value: function toString() {
      return "".concat(this.lowerBound.toString(), " ~ ").concat(this.upperBound.toString());
    }
    /**
     * Creates a Range from an array of two numbers.
     *
     * @public
     * @static
     * @function
     * @param  {Array.<Number>} values - Boundaries of the range.
     * @return {Range}
     */

  }], [{
    key: "fromArray",
    value: function fromArray(values) {
      if (!Array.isArray(values)) {
        throw new Error('Range#fromArray: values must be an Array of 2 numerical elements');
      }

      if (values.length !== 2) {
        throw new Error('Range#fromArray: values must be an Array of 2 numerical elements');
      }

      if (!values.every(function (value) {
        return typeof value === 'number';
      })) {
        throw new Error('Range#fromArray: values must be an Array of 2 numerical elements');
      }

      return new Range(Number(values[0]), Number(values[1]));
    }
    /**
     * Creates a Range from a string-based notation.
     *
     * @public
     * @static
     * @function
     * @param  {String} notedRange - A string representation of a Range,
     *                  using delimiters. Accepted formats: x,y x;y x:y x~y.
     * @return {Range}
     */

  }, {
    key: "fromNotation",
    value: function fromNotation(notedRange) {
      if (!notedRange) {
        throw new Error('Range#fromNotation: notedRange must use one of these formats: ' + 'x,y x;y x:y x~y');
      }

      if (typeof notedRange !== 'string') {
        throw new Error('Range#fromNotation: notedRange must use one of these formats: ' + 'x,y x;y x:y x~y');
      }

      var range = void 0;
      var allowedDelimiters = [',', ';', ':', '~'];

      if (!allowedDelimiters.some(function (delimiter) {
        return notedRange.includes(delimiter);
      })) {
        throw new Error('Range#fromNotation: notedRange must use one of these formats: ' + 'x,y x;y x:y x~y');
      }

      var delimiterFound = false;

      while (!delimiterFound) {
        var delimiter = allowedDelimiters.shift();
        delimiterFound = notedRange.includes(delimiter);

        if (delimiterFound) {
          var data = notedRange.split(delimiter).map(function (d) {
            return d.trim();
          });

          if (data.length !== 2) {
            throw new Error('Range#fromNotation: notedRange must use one of these formats: ' + 'x,y x;y x:y x~y');
          } else {
            range = Range.fromArray(data.map(function (d) {
              return Number(d);
            }));
          }
        }
      }

      if (!range) {
        throw new Error('Range#fromNotation: notedRange must use one of these formats: ' + 'x,y x;y x:y x~y');
      }

      return range;
    }
  }]);
  return Range;
}();

/**
 * @module xethya-extension-range
 * @namespace Xethya.Utils
 */
/**
 * Allows an object to be constrained by a range.
 * 
 * @mixin RangeableObject
 */

var RangeableObject = {
  /**
   * Gets the range definition for the object.
   * 
   * @public
   * @function getRange
   * @memberof RangeableObject
   * @instance
   * @return {Range}
   */
  getRange: function getRange() {
    return this._range;
  },

  /**
   * Sets the range definition for the object.
   * 
   * @public
   * @memberof RangeableObject
   * @instance
   * @method setRange
   * @param {number} [lowerBound=0] - The minimum value allowed by the range.
   * @param {number} [upperBound=Infinity] - The maximum value allowed by the range.
   */
  setRange: function setRange() {
    var lowerBound = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
    var upperBound = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : Infinity;
    this._range = new Range(lowerBound, upperBound);
  }
};

exports.Range = Range;
exports.RangeableObject = RangeableObject;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=xethya-extension-range.umd.js.map
