## Classes

<dl>
<dt><a href="#Range">Range</a></dt>
<dd><p>Represents a range of number, described through boundaries.</p>
</dd>
</dl>

## Mixins

<dl>
<dt><a href="#RangeableObject">RangeableObject</a></dt>
<dd><p>Allows an object to be constrained by a range.</p>
</dd>
</dl>

<a name="Range"></a>

## Range
**Kind**: global class  

* [Range](#Range)
    * [new Range(lowerBound, upperBound)](#new_Range_new)
    * _instance_
        * [.includes(value)](#Range+includes) ⇒ <code>Boolean</code>
        * [.toString()](#Range+toString) ⇒ <code>String</code>
    * _static_
        * [.fromArray(values)](#Range.fromArray) ⇒ [<code>Range</code>](#Range)
        * [.fromNotation(notedRange)](#Range.fromNotation) ⇒ [<code>Range</code>](#Range)


* * *

<a name="new_Range_new"></a>

### new Range(lowerBound, upperBound)
Instantiates a Range object.

**Throws**:

- <code>Error</code> If lowerBound and upperBound are the same.


| Param | Type | Description |
| --- | --- | --- |
| lowerBound | <code>Number</code> | The minimum number in this range. |
| upperBound | <code>Number</code> | The maximum number in this range. |


* * *

<a name="Range+includes"></a>

### range.includes(value) ⇒ <code>Boolean</code>
Checks if a value is in the defined range.

**Kind**: instance method of [<code>Range</code>](#Range)  
**Returns**: <code>Boolean</code> - true if in range, false otherwise.  
**Access**: public  

| Param | Type | Description |
| --- | --- | --- |
| value | <code>Number</code> | Value to compare. |


* * *

<a name="Range+toString"></a>

### range.toString() ⇒ <code>String</code>
Converts the Range object to a string representation.

**Kind**: instance method of [<code>Range</code>](#Range)  
**Access**: public  

* * *

<a name="Range.fromArray"></a>

### Range.fromArray(values) ⇒ [<code>Range</code>](#Range)
Creates a Range from an array of two numbers.

**Kind**: static method of [<code>Range</code>](#Range)  
**Access**: public  

| Param | Type | Description |
| --- | --- | --- |
| values | <code>Array.&lt;Number&gt;</code> | Boundaries of the range. |


* * *

<a name="Range.fromNotation"></a>

### Range.fromNotation(notedRange) ⇒ [<code>Range</code>](#Range)
Creates a Range from a string-based notation.

**Kind**: static method of [<code>Range</code>](#Range)  
**Access**: public  

| Param | Type | Description |
| --- | --- | --- |
| notedRange | <code>String</code> | A string representation of a Range,                  using delimiters. Accepted formats: x,y x;y x:y x~y. |


* * *

<a name="RangeableObject"></a>

## RangeableObject
Allows an object to be constrained by a range.

**Kind**: global mixin  

* [RangeableObject](#RangeableObject)
    * [.getRange()](#RangeableObject+getRange) ⇒ [<code>Range</code>](#Range)
    * [.setRange([lowerBound], [upperBound])](#RangeableObject+setRange)


* * *

<a name="RangeableObject+getRange"></a>

### rangeableObject.getRange() ⇒ [<code>Range</code>](#Range)
Gets the range definition for the object.

**Kind**: instance method of [<code>RangeableObject</code>](#RangeableObject)  
**Access**: public  

* * *

<a name="RangeableObject+setRange"></a>

### rangeableObject.setRange([lowerBound], [upperBound])
Sets the range definition for the object.

**Kind**: instance method of [<code>RangeableObject</code>](#RangeableObject)  
**Access**: public  

| Param | Type | Default | Description |
| --- | --- | --- | --- |
| [lowerBound] | <code>number</code> | <code>0</code> | The minimum value allowed by the range. |
| [upperBound] | <code>number</code> | <code>Infinity</code> | The maximum value allowed by the range. |


* * *

