/**
 * @module xethya-extension-range
 * @namespace Xethya.Utils
 */

import { Range } from './Range';

/**
 * Gets the range definition for the object.
 * 
 * @public
 * @function getRange
 * @memberof RangeableObject
 * @instance
 * @return {Range}
 */
const getRange = function () {
  return this._range;
};

/**
 * Sets the range definition for the object.
 * 
 * @public
 * @memberof RangeableObject
 * @instance
 * @method setRange
 * @param {number} [lowerBound=0] - The minimum value allowed by the range.
 * @param {number} [upperBound=Infinity] - The maximum value allowed by the range.
 */
const setRange = function (lowerBound = 0, upperBound = Infinity) {
  this._range = new Range(lowerBound, upperBound);
};

/**
 * Allows an object to be constrained by a range.
 * 
 * @mixin Rangeable
 */
export function Rangeable(target) {
  Object.assign(target.prototype, { 
    getRange, 
    setRange,
  });
};
